<?php
/**
 * Handles listing of events
 *
 * @licenseGPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author Chris Tregenza <chris@6d6fireball.com>
 */
 
if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
 
/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_indieplus_eventlist extends DokuWiki_Syntax_Plugin {
 
 	var $myData = null;
 
    function __construct() {
   		$this->helper = plugin_load ( 'helper', 'indieplus_dbhelper' );
    }
 
 
   /**
* Get an associative array with plugin info.
*
* <p>
* The returned array holds the following fields:
* <dl>
* <dt>author</dt><dd>Author of the plugin</dd>
* <dt>email</dt><dd>Email address to contact the author</dd>
* <dt>date</dt><dd>Last modified date of the plugin in
* <tt>YYYY-MM-DD</tt> format</dd>
* <dt>name</dt><dd>Name of the plugin</dd>
* <dt>desc</dt><dd>Short description of the plugin (Text only)</dd>
* <dt>url</dt><dd>Website with more information on the plugin
* (eg. syntax description)</dd>
* </dl>
* @param none
* @return Array Information about this plugin class.
* @public
* @static
*/
	function getInfo(){
		return array(
		'author' => 'Chris Tregenza',
		'email'  => 'chris@6d6fireball.com',
		'date'   => '2012-09-23',
		'name'   => 'Event List Plugin',
		'desc'   => 'Event list generation',
		'url'=> 'http://indieplus.org',
		);
	}
 
   /**
	* Get the type of syntax this plugin defines.
	*
	* @param none
	* @return String <tt>'substition'</tt> (i.e. 'substitution').
	* @public
	* @static
	*/
	function getType(){
		return 'substition';
	}
	 
	/**
	 * What kind of syntax do we allow (optional)
	 */
	//function getAllowedTypes() {
	//return array();
	//}
	 
	   /**
	* Define how this plugin is handled regarding paragraphs.
	*
	* <p>
	* This method is important for correct XHTML nesting. It returns
	* one of the following values:
	* </p>
	* <dl>
	* <dt>normal</dt><dd>The plugin can be used inside paragraphs.</dd>
	* <dt>block</dt><dd>Open paragraphs need to be closed before
	* plugin output.</dd>
	* <dt>stack</dt><dd>Special case: Plugin wraps other paragraphs.</dd>
	* </dl>
	* @param none
	* @return String <tt>'block'</tt>.
	* @public
	* @static
	*/
	//function getPType(){
	//return 'normal';
	//}
	 
	   /**
	* Where to sort in?
	*
	* @param none
	* @return Integer <tt>6</tt>.
	* @public
	* @static
	*/
	function getSort(){
		return 1040;
	}
 
 
	/**
	* Connect lookup pattern to lexer.
	*
	* @param $aMode String The desired rendermode.
	* @return none
	* @public
	* @see render()
	*/
	function connectTo($mode) {

// error_log("EventList -> connectTo");
		
		$this->Lexer->addSpecialPattern('<eventlist>',$mode,'plugin_indieplus_eventlist');
		$this->Lexer->addSpecialPattern('<eventlist\|.*>',$mode,'plugin_indieplus_eventlist');
	
	
	}
 
	//function postConnect() {
	//  $this->Lexer->addExitPattern('</TEST>','plugin_test');
	//}
 
 
   	/**
	* Handler to prepare matched data for the rendering process.
	*
	* <p>
	* The <tt>$aState</tt> parameter gives the type of pattern
	* which triggered the call to this method:
	* </p>
	* <dl>
	* <dt>DOKU_LEXER_ENTER</dt>
	* <dd>a pattern set by <tt>addEntryPattern()</tt></dd>
	* <dt>DOKU_LEXER_MATCHED</dt>
	* <dd>a pattern set by <tt>addPattern()</tt></dd>
	* <dt>DOKU_LEXER_EXIT</dt>
	* <dd> a pattern set by <tt>addExitPattern()</tt></dd>
	* <dt>DOKU_LEXER_SPECIAL</dt>
	* <dd>a pattern set by <tt>addSpecialPattern()</tt></dd>
	* <dt>DOKU_LEXER_UNMATCHED</dt>
	* <dd>ordinary text encountered within the plugin's syntax mode
	* which doesn't match any pattern.</dd>
	* </dl>
	* @param $aMatch String The text matched by the patterns.
	* @param $aState Integer The lexer state for the match.
	* @param $aPos Integer The character position of the matched text.
	* @param $aHandler Object Reference to the Doku_Handler object.
	* @return Integer The current lexer state for the match.
	* @public
	* @see render()
	* @static
	*/
	function handle($match, $state, $pos, &$handler){

// error_log("EventList -> handle");

		switch ($state) {
		  case DOKU_LEXER_ENTER : 
		break;
		  case DOKU_LEXER_MATCHED :
		break;
		  case DOKU_LEXER_UNMATCHED :
		break;
		  case DOKU_LEXER_EXIT :
		break;
		  case DOKU_LEXER_SPECIAL :
		  	$pos = strpos($match,"|");
		  	if ($pos !== false ) {
		  		$data = array("eventlist" => substr($match,$pos+1,strlen($match)-$pos-2));	
		  	} else {
		  		$data = array("eventlist" => NULL);	
		  	}
// error_log("EventList -> handle data ");
// error_log(var_export($data,1));		  	
		  	return $data;
		break;
		}
	return array();
	}
 
	/**
	* Handle the actual output creation.
	*
	* <p>
	* The method checks for the given <tt>$aFormat</tt> and returns
	* <tt>FALSE</tt> when a format isn't supported. <tt>$aRenderer</tt>
	* contains a reference to the renderer object which is currently
	* handling the rendering. The contents of <tt>$aData</tt> is the
	* return value of the <tt>handle()</tt> method.
	* </p>
	* @param $aFormat String The output format to generate.
	* @param $aRenderer Object A reference to the renderer object.
	* @param $aData Array The data created by the <tt>handle()</tt>
	* method.
	* @return Boolean <tt>TRUE</tt> if rendered successfully, or
	* <tt>FALSE</tt> otherwise.
	* @public
	* @see handle()
	*/
	
	function render($mode, &$renderer, $data) {
		if($mode == 'xhtml') {
			$renderer->doc .= $this->buildList($data);
			return true;
		}
		return false;
	}

	function buildList( $data ) {
	
		$dbResult = $this->getData( $data['eventlist'] );
		$headers = '<div class="eventlist"><div class="eventlistheaders">';
		$rows = '<div class="eventlistrows">';

		if (count($dbResult) > 0 ) {
			foreach($dbResult[0] as $field => $value ) {
				$headers .= '<span class="'.$field.'_eventlist_header">'.$field.'</span>'; 	
			}
		}
		$headers .= "</div>";

		foreach($dbResult as $row ) {
			$rows .= '<div class="eventlistrow">';
			foreach($row as $field => $value) {
				if ( $field == 'eventid' ) {
					$rows .= '<span class="'.$field.'_eventlist">';
					$rows .= '<a href="/doku.php?id=events:indie2:'.$value.'">'.$value.'</a></span>'; 					
				} else {
					$rows .= '<span class="'.$field.'_eventlist">'.$value.'</span>'; 					
				}
			} 
			$rows .= '</div>';	
		}
		$rows .= '</div>';

		$rValue = $headers.$rows.'</div>';

// error_log($rValue);

		return $rValue;

	}
	
    function getData( $condition ) 
    {
		$sql = $this->getConf('eventlist_query');

		if ( isset($condition) ) {
            $sql = $this->getConf('eventlist_query');
			$sql .= " where ".$condition;
        }
			
		list($dbtype, $Server,$User,$Pass,$Database) =  explode(":",$this->getConf('indieplusDB'));
        try{  
			$rs = $this->helper->_mysql($Server,$User,$Pass,$Database,$sql);
 		}catch(Exception $ex){
        	$sResponse = $this->_error($this->aString["problem"]);
			return $sResponse;
        }    

	 	return $rs;        
			
	}
	
}
 
//Setup VIM: ex: et ts=4 :
?>
