<?php
/**
 * Handles replacement of field names on indieplus event pages
 *
 * @licenseGPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author Chris Tregenza <chris@6d6fireball.com>
 */
 
if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../../').'/');
if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');
 
/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_indieplus_events extends DokuWiki_Syntax_Plugin {
 
 	var $myData = null;
 
    function __construct() {
   		$this->helper = plugin_load ( 'helper', 'indieplus_dbhelper' );
    }
 
 
   /**
* Get an associative array with plugin info.
*
* <p>
* The returned array holds the following fields:
* <dl>
* <dt>author</dt><dd>Author of the plugin</dd>
* <dt>email</dt><dd>Email address to contact the author</dd>
* <dt>date</dt><dd>Last modified date of the plugin in
* <tt>YYYY-MM-DD</tt> format</dd>
* <dt>name</dt><dd>Name of the plugin</dd>
* <dt>desc</dt><dd>Short description of the plugin (Text only)</dd>
* <dt>url</dt><dd>Website with more information on the plugin
* (eg. syntax description)</dd>
* </dl>
* @param none
* @return Array Information about this plugin class.
* @public
* @static
*/
	function getInfo(){
		return array(
		'author' => 'Chris Tregenza',
		'email'  => 'chris@6d6fireball.com',
		'date'   => '2012-09-14',
		'name'   => 'Event Plugin',
		'desc'   => 'Event field replacement',
		'url'=> 'http://indieplus.org',
		);
	}
 
   /**
	* Get the type of syntax this plugin defines.
	*
	* @param none
	* @return String <tt>'substition'</tt> (i.e. 'substitution').
	* @public
	* @static
	*/
	function getType(){
		return 'substition';
	}
	 
	/**
	 * What kind of syntax do we allow (optional)
	 */
	//function getAllowedTypes() {
	//return array();
	//}
	 
	   /**
	* Define how this plugin is handled regarding paragraphs.
	*
	* <p>
	* This method is important for correct XHTML nesting. It returns
	* one of the following values:
	* </p>
	* <dl>
	* <dt>normal</dt><dd>The plugin can be used inside paragraphs.</dd>
	* <dt>block</dt><dd>Open paragraphs need to be closed before
	* plugin output.</dd>
	* <dt>stack</dt><dd>Special case: Plugin wraps other paragraphs.</dd>
	* </dl>
	* @param none
	* @return String <tt>'block'</tt>.
	* @public
	* @static
	*/
	//function getPType(){
	//return 'normal';
	//}
	 
	   /**
	* Where to sort in?
	*
	* @param none
	* @return Integer <tt>6</tt>.
	* @public
	* @static
	*/
	function getSort(){
		return 1040;
	}
 
 
	/**
	* Connect lookup pattern to lexer.
	*
	* @param $aMode String The desired rendermode.
	* @return none
	* @public
	* @see render()
	*/
	function connectTo($mode) {
		// Am I in the right namespace
		global $ID;
		
		$myid = $ID;
		if ( substr($myid,0,1) != ":" ) {
			$myid = ":".$myid;	
		}
// error_log("events: ".$myid." / ".$this->getConf('eventnamespace')." / ".strpos($myid, $this->getConf('eventnamespace') ) );
				
		if (strpos($myid, $this->getConf('eventnamespace')) === 0 ) {
			// We are in the name space
			$fields = $this->getConf('eventfieldnames');
			$fieldArray = explode("|",$fields);
			foreach( $fieldArray as $field ) {
// error_log("events: field ".$field);		
	  			$this->Lexer->addSpecialPattern('<'.$field.'>',$mode,'plugin_indieplus_events');
			}
			
			
		}
		
	
	
	}
 
	//function postConnect() {
	//  $this->Lexer->addExitPattern('</TEST>','plugin_test');
	//}
 
 
   	/**
	* Handler to prepare matched data for the rendering process.
	*
	* <p>
	* The <tt>$aState</tt> parameter gives the type of pattern
	* which triggered the call to this method:
	* </p>
	* <dl>
	* <dt>DOKU_LEXER_ENTER</dt>
	* <dd>a pattern set by <tt>addEntryPattern()</tt></dd>
	* <dt>DOKU_LEXER_MATCHED</dt>
	* <dd>a pattern set by <tt>addPattern()</tt></dd>
	* <dt>DOKU_LEXER_EXIT</dt>
	* <dd> a pattern set by <tt>addExitPattern()</tt></dd>
	* <dt>DOKU_LEXER_SPECIAL</dt>
	* <dd>a pattern set by <tt>addSpecialPattern()</tt></dd>
	* <dt>DOKU_LEXER_UNMATCHED</dt>
	* <dd>ordinary text encountered within the plugin's syntax mode
	* which doesn't match any pattern.</dd>
	* </dl>
	* @param $aMatch String The text matched by the patterns.
	* @param $aState Integer The lexer state for the match.
	* @param $aPos Integer The character position of the matched text.
	* @param $aHandler Object Reference to the Doku_Handler object.
	* @return Integer The current lexer state for the match.
	* @public
	* @see render()
	* @static
	*/
	function handle($match, $state, $pos, &$handler){
		switch ($state) {
		  case DOKU_LEXER_ENTER : 
		break;
		  case DOKU_LEXER_MATCHED :
		break;
		  case DOKU_LEXER_UNMATCHED :
		break;
		  case DOKU_LEXER_EXIT :
		break;
		  case DOKU_LEXER_SPECIAL :
		  	$fieldname = substr($match,1,strlen($match)-2);
		  	return array($fieldname);
		break;
		}
	return array();
	}
 
	/**
	* Handle the actual output creation.
	*
	* <p>
	* The method checks for the given <tt>$aFormat</tt> and returns
	* <tt>FALSE</tt> when a format isn't supported. <tt>$aRenderer</tt>
	* contains a reference to the renderer object which is currently
	* handling the rendering. The contents of <tt>$aData</tt> is the
	* return value of the <tt>handle()</tt> method.
	* </p>
	* @param $aFormat String The output format to generate.
	* @param $aRenderer Object A reference to the renderer object.
	* @param $aData Array The data created by the <tt>handle()</tt>
	* method.
	* @return Boolean <tt>TRUE</tt> if rendered successfully, or
	* <tt>FALSE</tt> otherwise.
	* @public
	* @see handle()
	*/
	
	function render($mode, &$renderer, $data) {
		if($mode == 'xhtml') {
			$renderer->doc .= $this->myData($data[0]);
			return true;
		}
		return false;
	}

	function myData( $fieldName ) {
	
		if ( !isset($this->myData) ) {
			$this->getMyData();	
		}
// error_log("myData myData[".$fieldName."] ".$this->myData[$fieldName]);

		if ( !isset($this->myData[$fieldName] ) ) {
			$rValue = "!!No Data!!";			
		} else {
			switch ( $fieldName ) {
				case 'datetime':
					$tz = new DateTimeZone('GMT');
					$dt = new DateTime($this->myData['datetime'], $tz);
					$time = '<span class="datetime_gmt">GMT: '.$dt->format( 'D, d M Y H:i' ).'</span>';

					$tz = new DateTimeZone('Europe/London');
					$dt->setTimeZone($tz);
					$time .= '<span class="datetime_uk">'.$dt->format( ' e H:i ( O )' ).'</span>';
						
					$tz = new DateTimeZone('US/Eastern');
					$dt->setTimeZone($tz);
					$time .= '<span class="datetime_est">'.$dt->format( ' e H:i ( O )' ).'</span>';

					$tz = new DateTimeZone('US/Pacific');
					$dt->setTimeZone($tz);
					$time .= '<span class="datetime_wst">'.$dt->format( ' e H:i ( O )' ).'</span>';

					$tz = new DateTimeZone('Australia/Sydney');
					$dt->setTimeZone($tz);
					$time .= '<span class="datetime_awst">'.$dt->format( ' e H:i ( O )' ).'</span>';
					
					$rValue = $time;
					break;
				case 'eventlink':
					$rValue = '<a class="urlextern" href="'.$this->myData['eventlink'].'">'.$this->myData['eventname'].'</a>';
					break;
				case 'eventhostname':
					$rValue = '<a class="urlextern" href="'.$this->myData['hostprofileurl'].'">'.$this->myData[$fieldName].'</a>';
					break;
				default:
					$rValue = $this->myData[$fieldName];	
					break;
			}
		}
		
		return '<span class="eventField" id="'.$fieldName.'">'.$rValue.'</span>';
	}
	
	function getMyData() {
		global $ID;
		$temp = split(":", $ID);
		$mypage = $temp[count($temp)-1];
		
		$sql = $this->getConf('eventquery');
// error_log("getMyData: sql = ".$sql);		
		$sql = str_replace("%id%", $mypage, $sql);
// error_log("getMyData: sql(2) = ".$sql);		
			
		list($dbtype, $Server,$User,$Pass,$Database) =  explode(":",$this->getConf('indieplusDB'));
        try{  
			$rs = $this->helper->_mysql($Server,$User,$Pass,$Database,$sql);
 		}catch(Exception $ex){
        	$sResponse = $this->_error($this->aString["problem"]);
// error_log("Query Exception");
// error_log(var_export($ex,1));
			return $sResponse;
        }    

// error_log("getMyData rs");
// error_log(var_export($rs,1));
 
        if ($rs !== false && isset( $rs[0] ) ){
        	$this->myData =  $rs[0];
        }
// error_log("getMyData myData");
// error_log(var_export($this->myData,1));
        
			
	}
	
}
 
//Setup VIM: ex: et ts=4 enc=utf-8 :
?>

