<?php
/**
 *
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Christoph Lang <calbity@gmx.de>
 */
 
// based on http://www.dokuwiki.org/plugin:tutorial
 
/**
 *
 * Usage:
 * [[mysql:server:username:password:database|query|refresh]]
 * [[mssql:server:username:password:database|query|refresh]]
 * [[oracle:server:username:password:database|query|refresh]]
 * [[sqlite:unused:unused:unused:path to sqlite3 database|query|refresh]]
 * [[sqlcsv:unused:unused:unused:path to csv file|delimiter|refresh]]
 * [[sqlaccess:unused:unused:password:path to mdb file|query|refresh]]
 * [[postgresql:server:username:password:database|query|refresh]]
 */
 
/**
 * Disabled, to use that class in standalone mode...
 * // must be run within DokuWiki
 */
 
if (!defined('DOKU_INC')) die();
if (!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN', DOKU_INC . 'lib/plugins/');
require_once(DOKU_PLUGIN . 'syntax.php');
 
 
/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_indieplus_sqlcomp extends DokuWiki_Syntax_Plugin {
 
    private $sPath;
    
    
    
//    private $sConfig = "lib/plugins/sqlcomp/config.php";
 
    /*
    Layout
    */
    private $aMessages = array(
                              "error" => "<div id=\"error\" style=\"text-align:center; font-weight: bold; border: 2px solid #0f0;background-color: #f00; padding: 5px; margin: 5px\">%text%</div>\n",
                              "message" => "<div id=\"difference\" style=\"text-align:center; font-weight: bold; border: 2px solid #fd0;background-color: #ffd; padding: 5px; margin: 5px\">%text%</div>\n",
                              "pre" => "<table class=\"inline\">\n",
                              "post" => "</table>\n",
                              "th" => "<th class=\"row%number%\" style=\"%type%\">%text%</th>",
                              "td" => "<td class=\"col%number%\" style=\"%type%\">%text%</td>",
                              "tr" => "<tr class=\"row%number%\" style=\"%type%\">%text%</tr>\n",
                              "same" => "",
                              "new" => "border:2px solid green;",
                              "deleted" => "border:2px solid red;",
                              "changed" => "border:2px solid blue;"
                              );
    /*
    Default Language - German
    */
    private $aString = array(
                            //Number of affected Rows
                            "affected" => "Number of affected Rows",
                            //This Database Type is not yet Supported...
                            "nohandler" => "This Database Type is not yet Supported...",
                            //There are some differences in the table!
                            "difference" => "There are some differences in the table!",
                            //Everything is allright.
                            "same" => "Everything is allright.",
                            //The resultset is empty.
                            "empty" => "The resultset is empty.",
                            //An unkown error occured!
                            "problem" => "An unkown error occured!",
                            //Cache is displayed, but new data could not be retrieved.
                            "cache" => "Cache is displayed, but new data could not be retrieved.",
                            //Cache was refreshed, or table was collected for the first time.
                            "first" => "Cache was refreshed, or table was collected for the first time.",
                            //New data could not be retrieved.
                            "connection" => "New data could not be retrieved.",
                            //The data is not valid. Please review your connection settings!
                            "wrong" => "The data is not valid. Please review your connection settings!"
                            );
 
    private $defaultRefresh = 1;
 
    function getInfo() {
        return array(
        'author'  => 'Christoph Lang',
        'email'   => 'calbity@gmx.de',
        'date'    => '2008-07-10',
        'name'    => 'SQLCOMP Plugin',
        'desc'    => 'This plugin let you display reultsets from various databases and show changes.',
        'url'     => 'http://www.google.de'
        );
    }
 
    public function query($query){
      $temp = null;
      $data = $this->handle($query,"","",$temp);
      return $this->_query($data,"csv");       
    }
 
    public function __construct(){      
 		global $conf;
 
 		$this->sPath = $conf['savedir']."/sql";
 		
 		$this->helper = plugin_load ( 'helper', 'indieplus_dbhelper' );
    
    }
 
    private function _error($text){
      return str_replace("%text%",$text,$this->aMessages["error"]);      
    }
 
    private function _message($text){
      return str_replace("%text%",$text,$this->aMessages["message"]);
 
    }
 
 
 

    private function _debug($data){
 
        $sResponse = "";
        foreach($data as $key => $value)
            $sResponse .= "".$key . "=> " .$value ."<br/>\n";            
        return $sResponse;
 
    }
 
    private function _verifyInput($data){
      if(!is_array($data))
        return false;
      if(count($data) != 7)
        return false;
      return true;    
    }
 
    private function _load($filename){
 
        $Cache = null;
        $Update = true;
        if(file_exists($filename)){
          $Cache = file_get_contents($filename);  
          $Cache = unserialize($Cache);
 
          $Update = $Cache["Update"];
          if(time() > $Update)
            $Update = true;
          else
            $Update = false;
          $Cache = $Cache["Table"];    
        }        
 
        return array($Update,$Cache);
    }
 
    private function _save($filename,$rs,$timestamp){
        $timestamp = (time() + ($timestamp*60));
        $Cache["Update"] = $timestamp;
        $Cache["Table"] = $rs;
 
        $Cache = serialize($Cache);
 
        $handle = fopen($filename,"w");
        fwrite($handle,$Cache);
        fclose($handle);
 
 
    }
    private function array2csv($data){
 
      $sResponse = "";
 
      $keys = array_keys($data[0]);
      $sResponse .= implode(";",$keys)."\n";
      foreach($data as $row)
        $sResponse .= implode(";",$row)."\n";
 
 
      return $sResponse;
 
    }
 
 
    private function _query($data,$type=null) {      

        if(!$this->_verifyInput($data))
          return $this->_error($this->aString["wrong"]);
 
        if(!is_dir($this->sPath))
          mkdir($this->sPath);
 
        $filename = $this->sPath.md5($data[0].$data[1].$data[2].$data[3].$data[4].$data[5]);
 
        $Cache = $this->_load($filename);
        $Update = true;
        if(is_array($Cache)){
          $Update = $Cache[0];
          $Cache = $Cache[1];
        }
 
        try{  
          switch($data[0]){
            case "mysql": $rs = $this->helper->_mysql($data[1], $data[2], $data[3],$data[4],$data[5]); break;
            case "mssql": $rs = $this->helper->_mssql($data[1], $data[2], $data[3],$data[4],$data[5]); break;
            case "oracle": $rs = $this->helper->_oracle($data[1], $data[2], $data[3],$data[4],$data[5]); break;
            case "sqlite": $rs = $this->helper->_sqlite($data[1], $data[2], $data[3],$data[4],$data[5]); break;
            case "sqlaccess": $rs = $this->helper->_sqlaccess($data[1], $data[2], $data[3],$data[4],$data[5]); break;
            case "postgresql": $rs = $this->helper->_postgresql($data[1], $data[2], $data[3],$data[4],$data[5]); break;
            case "sqlcsv": $rs = $this->helper->_sqlcsv($data[1], $data[2], $data[3],$data[4],$data[5]); break;
            default: return $this->_error($this->aString["nohandler"]);
          }
        }catch(Exception $ex){
          $sResponse = $this->_error($this->aString["problem"]);
error_log("Query Exception");
error_log(var_export($ex,1));
          if(isset($Cache)){
            $sResponse = $this->_print($Cache);    
            $sResponse .= $this->_error($this->aString["cache"]);
          }
          return $sResponse;
        }    
error_log("_query rs");
error_log(var_export($rs,1));
 
        if ($rs === false || count($rs) == 0){
          return $this->_error($this->aString["empty"] );
        }
 
        if(isset($type) && $type == "csv")
          return $this->array2csv($rs);
 
        $difference = $this->_difference($Cache,$rs);
        $sResponse = $difference[0];    
 
        if($Update && isset($rs)){
          $this->_save($filename,$rs,$data[6]);      
 
        }  
 
        $sResponse .= $difference[1];      
 
 
        return $sResponse;
    }
    function _print($array){
 
        $i = 0;
 
        $th = "";
        $td = "";
        $tr = "";
        if(!isset($array[0])) {
error_log("Print Problem");
error_log(var_export($array, 1));
            return $this->_error($this->aString["problem"]);
        }
	     
        $temp = array_keys($array[0]);
        foreach($temp as $column){
          if($column == "type")
            continue;  
          $th .= str_replace(array("%number%","%text%","%type%"),array(0,$column,""),$this->aMessages["th"]);      
        }
        $tr = str_replace(array("%number%","%text%","%type%"),array(0,$th,""),$this->aMessages["tr"]);
 
        foreach($array as $row) {
 
          $j = 0;
          $td = "";
          if(!isset($row["type"]))
            $row["type"] = $this->aMessages["same"];
 
          foreach($row as $key => $Value){          
            if($key == "type")
              continue;  
            $td .= str_replace(array("%number%","%text%","%type%"),array($j,$Value,$row["type"]),$this->aMessages["td"]);
            $j++;            
          }
          $tr .= str_replace(array("%number%","%text%","%type%"),array($i,$td,$row["type"]),$this->aMessages["tr"]);
          $i++;          
        }
 
        $sResponse = $this->aMessages["pre"];
        $sResponse .= $tr;        
        $sResponse .= $this->aMessages["post"];
 
        return $sResponse;
 
 
    }
 
 
    function _difference($Cache,$New){
 
            if($New == $Cache){
              return array($this->_print($New),"");
              return array($this->_print($New),$this->_message($this->aString["same"]));
            }
 
            if(!isset($New) && isset($Cache))
              return array($this->_print($Cache),$this->_message($this->aString["difference"]));
 
            if(isset($New) && !isset($Cache))
              return array($this->_print($New),$this->_message($this->aString["first"]));
 
            if(count($New) <= 0)
              return array($this->_print($Cache),$this->_message($this->aString["connection"]));
 
            $Max = count($Cache);
            if(count($New) > count($Cache))
              $Max = count($New);
 
            $PrintArray = array();        
 
            for($i=0; $i < $Max; $i++){
              if(isset($Cache[$i]) && !isset($New[$i]))
                $PrintArray[] = array_merge($Cache[$i],array("type" => $this->aMessages["deleted"]));
 
              if(!isset($Cache[$i]) && isset($New[$i]))
                $PrintArray[] = array_merge($New[$i],array("type" => $this->aMessages["new"]));
 
              if(isset($Cache[$i]) && isset($New[$i])){
                if($Cache[$i] != $New[$i]){
                  $PrintArray[] = array_merge($Cache[$i],array("type" => $this->aMessages["changed"]));
                  $PrintArray[] = array_merge($New[$i],array("type" => $this->aMessages["changed"]));
                }else
                  $PrintArray[] = array_merge($New[$i],array("type" => $this->aMessages["same"]));
 
              }                
 
            }
 
            return array($this->_print($PrintArray),$this->_message($this->aString["difference"]));
 
    }
 
    function connectTo($mode) {
        $this->Lexer->addSpecialPattern('\[\[mysql\:.*?\]\]', $mode, 'plugin_indieplus_sqlcomp');
        $this->Lexer->addSpecialPattern('\[\[mssql\:.*?\]\]', $mode, 'plugin_indieplus_sqlcomp');
        $this->Lexer->addSpecialPattern('\[\[oracle\:.*?\]\]', $mode, 'plugin_indieplus_sqlcomp');
        $this->Lexer->addSpecialPattern('\[\[sqlite\:.*?\]\]', $mode, 'plugin_indieplus_sqlcomp');
        $this->Lexer->addSpecialPattern('\[\[sqlaccess\:.*?\]\]', $mode, 'plugin_indieplus_sqlcomp');
        $this->Lexer->addSpecialPattern('\[\[postgresql\:.*?\]\]', $mode, 'plugin_indieplus_sqlcomp');
        $this->Lexer->addSpecialPattern('\[\[sqlcsv\:.*?\]\]', $mode, 'plugin_indieplus_sqlcomp');
 

        $this->Lexer->addSpecialPattern('\[\[indieplusDB.*?\]\]', $mode, 'plugin_indieplus_sqlcomp');
 

    }
    function _createConfig(){  
/* 
        $sContent = "";
        $sContent .= "<?php\n";
        $sContent .= "//Sample Configfile\n";
        $sContent .= "//Add as many servers as you want here...\n";
        $sContent .= '$sqlcomp["localhost"] = "mysql:localhost:root::information_schema";';
        $sContent .= '$sqlcomp["sampleconnection"] = "sqltype:servername:username:password:database";';
        $sContent .= "\n?>\n";
 
        $handle = fopen($this->sConfig,"w");
        fwrite($handle,$sContent);
        fclose($handle);
*/ 
    }
 
    function getType() { return 'substition'; }
 
    function getSort() { return 267; }
 
    function handle($match, $state, $pos, &$handler) {
 
        $temp = $match;
        $match = substr($match,2,-2);
 
        $MyData = array();       
        $match = explode("|",$match);   
 
        if($match[0] == 'indieplusDB') {
            $match[0] = $this->getConf('indieplusDB');
        }
 
        $MyData =  explode(":",$match[0]);
        $MyData[] = $match[1];    
 
        if(isset($match[2]))
          $MyData[] = $match[2];    
        else
          $MyData[] = $this->defaultRefresh;  
 
        for($i=0;$i < 5; $i++)
          $MyData[$i] = str_replace(" ", ":",$MyData[$i]);

// error_log("XXX 2");
// error_log(var_export($MyData,1));  

        return $MyData;
 
    }
 
    function render($mode, &$renderer, $data) {
 
        if ($mode == 'xhtml') {          
            $renderer->doc .= $this->_query($data);               
            return true;
        }
        return false;
    }
}